'===============================================================================
' Copyright 2014 Alachisoft.  All rights reserved.
'===============================================================================


Imports System
Imports Alachisoft.NCache.Web.Caching
Imports Alachisoft.NCache.Web
Imports Alachisoft.NCache.Runtime
Imports Alachisoft.NCache.Runtime.Dependencies
Imports System.Configuration
Imports System.Collections

 Namespace NCacheSample

    ''' <summary>
    ''' Summary description for NCacheUtil.
    ''' </summary>

    Public Class NCacheUtil

        <Flags()> Public Enum InvalidateType As Integer
            RelatedCollection = 1
            QueryCollectionPK = 2
            QueryCollection = 4
            All = RelatedCollection Or QueryCollectionPK Or QueryCollection
        End Enum
        Public Shared _cache As Cache



        Public Shared Sub InitCache(ByVal inProc As Boolean, ByVal cacheId As String)
            If cacheId Is Nothing Then
                cacheId = "myCache"
            End If
            If inProc Then
                _cache = NCache.InitializeCache(cacheId)
            Else
                _cache = NCache.InitializeCache(cacheId)
            End If
        End Sub
        Public Shared Sub ClearCache()
            _cache.Clear()
        End Sub
        Public Shared Sub DisposeCache()
            If Not _cache Is Nothing Then
                _cache.Dispose()
            End If
        End Sub

        Public Shared Function Load(ByVal key As Object) As Object

            If key Is Nothing Then Exit Function

            Return _cache.Get(Convert.ToString(key))

        End Function

        Public Shared Function Store(ByVal key As String, ByVal objValue As Object, ByVal dependencyKeys As ArrayList) As Object
            Dim objTemp As Object

            If (Not (dependencyKeys Is Nothing) AndAlso dependencyKeys.Count <> 0) Then
                Dim cacheKeys(dependencyKeys.Count - 1) As String '= New String[dependencyKeys.Count)

                Array.Copy(dependencyKeys.ToArray(), cacheKeys, dependencyKeys.Count)
                objTemp = _cache.Add(key, objValue, New CacheDependency(Nothing, cacheKeys), Cache.NoAbsoluteExpiration, Cache.NoSlidingExpiration, CacheItemPriority.Default)

            Else

                objTemp = _cache.Add(key, objValue, Nothing, Cache.NoAbsoluteExpiration, Cache.NoSlidingExpiration, CacheItemPriority.Default)

            End If

            Store = objTemp
        End Function

        Public Shared Function Store(ByVal key As String, ByVal objValue As Object, ByVal dependencyKeys As ArrayList, ByVal slidingExpiration As TimeSpan) As Object
            Dim objTemp As Object

            If (Not (dependencyKeys Is Nothing) And dependencyKeys.Count <> 0) Then
                Dim cacheKeys(dependencyKeys.Count - 1) As String

                Array.Copy(dependencyKeys.ToArray(), cacheKeys, dependencyKeys.Count)
                objTemp = _cache.Add(key, objValue, New CacheDependency(Nothing, cacheKeys), Cache.NoAbsoluteExpiration, slidingExpiration, CacheItemPriority.Default)

            Else

                objTemp = _cache.Add(key, objValue, Nothing, Cache.NoAbsoluteExpiration, slidingExpiration, CacheItemPriority.Default)

            End If

            Store = objTemp
        End Function

        Public Shared Function Store(ByVal key As String, ByVal objValue As Object, ByVal dependencyKeys As ArrayList, ByVal absoluteExpiration As DateTime) As Object
            Dim objTemp As Object

            If (Not (dependencyKeys Is Nothing) AndAlso dependencyKeys.Count <> 0) Then
                Dim cacheKeys(dependencyKeys.Count - 1) As String

                Array.Copy(dependencyKeys.ToArray(), cacheKeys, dependencyKeys.Count)
                objTemp = _cache.Add(key, objValue, New CacheDependency(Nothing, cacheKeys), absoluteExpiration, Cache.NoSlidingExpiration, CacheItemPriority.Default)

            Else

                objTemp = _cache.Add(key, objValue, Nothing, absoluteExpiration, Cache.NoSlidingExpiration, CacheItemPriority.Default)

            End If

            Store = objTemp
        End Function

        Public Shared Function Insert(ByVal key As String, ByVal objValue As Object, ByVal dependencyKeys As ArrayList, ByVal absoluteExpiration As DateTime)
            If (Not (dependencyKeys Is Nothing) AndAlso dependencyKeys.Count <> 0) Then
                Dim cacheKeys(dependencyKeys.Count - 1) As String
                Array.Copy(dependencyKeys.ToArray(), cacheKeys, dependencyKeys.Count)
                _cache.Insert(key, objValue, New CacheDependency(Nothing, cacheKeys), absoluteExpiration, Cache.NoSlidingExpiration, CacheItemPriority.Default)
            Else
                _cache.Insert(key, objValue, Nothing, absoluteExpiration, Cache.NoSlidingExpiration, CacheItemPriority.Default)
            End If
        End Function

        'Public Shared Function Insert(ByVal key As String, ByVal objValue As Object, ByVal dependency As CacheDependency, ByVal slidingExpiration As TimeSpan)
        '    NCache.Cache.Insert(key, objValue, Nothing, Cache.NoAbsoluteExpiration, slidingExpiration, CacheItemPriority.Default, Nothing)
        'End Function

        Public Shared Function Insert(ByVal key As String, ByVal objValue As Object, ByVal dependency As CacheDependency)
            _cache.Insert(key, objValue, Nothing, Cache.NoAbsoluteExpiration, Cache.NoSlidingExpiration, CacheItemPriority.Default)
        End Function

        Public Shared Function Remove(ByVal key As String) As Boolean
            Dim isRemoved As Object

            isRemoved = _cache.Remove(key)
            If (Not isRemoved Is Nothing) Then
                Return True
            Else
                Return False
            End If
        End Function

        Public Shared Function Remove(ByVal keys() As String)
            Dim i As Integer
            For i = 0 To keys.Length - 1
                _cache.Remove(keys(i))
            Next
        End Function

#Region "[Cache Keys Implementation]"

        Public Shared Function GetObjectKey(ByVal objName As String, ByVal ParamArray pkAttrs() As String) As String

            Dim objKey As String
            Dim i As Int16

            objKey = objName & "."

            For i = 0 To pkAttrs.Length - 2
                objKey += pkAttrs(i) & "."
            Next

            objKey += pkAttrs(i)

            Return objKey
        End Function

        Public Shared Function GetQueryKey(ByVal objName As String, ByVal qryName As String, ByVal ParamArray pkAttrs() As String) As String

            Dim objKey As String
            Dim i As Int16

            objKey = objName & "."

            If Not pkAttrs Is Nothing Then

                For i = 0 To pkAttrs.Length - 1
                    objKey += pkAttrs(i) & "."
                Next

            End If

            objKey += qryName

            Return objKey
        End Function

        Public Shared Function GetRelationKey(ByVal objName As String, ByVal relName As String, ByVal ParamArray pkAttrs() As String) As String

            Dim objKey As String
            Dim i As Int16

            objKey = objName & "."

            For i = 0 To pkAttrs.Length - 1
                objKey += pkAttrs(i) & "."
            Next

            objKey += relName

            Return objKey
        End Function
#End Region

        Protected Overrides Sub Finalize()
            MyBase.Finalize()
        End Sub
    End Class


End Namespace
