'===============================================================================
' Copyright 2014 Alachisoft.  All rights reserved.
'===============================================================================


Imports NCacheSample
Imports System.Xml

#Region "Class Customer Info"
'Stores information about customer
Class CustomerInfo
    'Private Members
    Private _customerId As String
    Private _customerContactName As String
    Public Sub New(ByVal customerId As String, ByVal name As String)
        _customerId = customerId
        _customerContactName = name
    End Sub
    'Gets/Sets the id of the customer
    Public Property Id() As String
        Get
            Return _customerId
        End Get
        Set(ByVal Value As String)
            _customerId = Value
        End Set
    End Property
    'Gets/Sets the name of the customer
    Public Property name() As String
        Get
            Return _customerContactName
        End Get
        Set(ByVal Value As String)
            _customerContactName = Value
        End Set
    End Property
End Class
#End Region
#Region "Class Sales Person"
' Stores information about the sales person
Class SalesPerson
    'Private Members
    Private _salesPersonId As Int64
    Private _salesPerson As String
    Public Sub New(ByVal salesPersonId As Int64, ByVal name As String)
        _salesPersonId = salesPersonId
        _salesPerson = name
    End Sub
    'Gets/Sets the ID of the sales person
    Public Property Id() As Int64
        Get
            Return _salesPersonId
        End Get
        Set(ByVal Value As Int64)
            _salesPersonId = Value
        End Set
    End Property
    'Gets/Sets the name of the sales person
    Public Property name() As String
        Get
            Return _salesPerson
        End Get
        Set(ByVal Value As String)
            _salesPerson = Value
        End Set
    End Property
End Class
#End Region
#Region "Class Document"
Public Class Document
    'Private Members

    'Current customer
    Private _customer As Customers
    'Collection of all the customers
    Private _customerCollection As CustomersCollection
    'Current order of the customer
    Private _order As Orders
    ' Flag for enabling cache
    Private _cacheEnabled As Boolean
    'Current order no
    Private _orderNo As Int64
    'Collection of the salesperson
    Private _salesPerson As EmployeesCollection
    'Collection of the products
    Private _products As ProductsCollection
    'Id of the cache
    Private _cacheId As String
    'Flag for cahce mode
    Private _inproc As Boolean
    Private _expirationInterval As Integer

#Region "Properties"
    'Gets the order of the customer
    Public ReadOnly Property Order() As Orders
        Get
            Return _order
        End Get
    End Property
    'Gets the total number of ordes for a cutomer
    Public ReadOnly Property OrdersCount() As Int32
        Get
            If Not (_customer Is Nothing) Then
                If Not (_customer.Orders Is Nothing) Then
                    Return _customer.Orders.Items.Count
                End If
            End If
            Return 0
        End Get

    End Property
    'Gets whether the order is new 
    Public ReadOnly Property IsOrderNewOrder()
        Get
            If Not _order Is Nothing Then
                Return _order.IsNew
            End If
            Return False
        End Get
    End Property
    'Gets the number of the current order
    Public ReadOnly Property OrderNo() As Int64
        Get
            If Not _customer.Orders Is Nothing Then
                If _customer.Orders.Count > 0 Then
                    Return Me._orderNo + 1
                End If
            End If
            Return 0
        End Get
    End Property
    'Gets/Sets the flag whether to use the cache or not
    Public Property CacheEnabled() As Boolean
        Get
            Return _cacheEnabled
        End Get
        Set(ByVal Value As Boolean)
            _cacheEnabled = Value
        End Set
    End Property
    'Gets/Sets the flag for cahce mode
    Public Property IsInproc() As Boolean
        Get
            Return _inproc
        End Get
        Set(ByVal Value As Boolean)
            _inproc = Value
        End Set
    End Property
    Public Property UseCache() As String
        Get
            Return _cacheId
        End Get
        Set(ByVal Value As String)
            _cacheId = Value
        End Set
    End Property
#End Region
#Region "Customer Related Operations"
    'Gets the customer data from NCache if enabled otherwise from the databse
    Public Function GetCustomer(ByVal customerID As String) As Customers
        Dim customerFactory As CustomersFactory
        Try
            customerFactory = New CustomersFactory
            _customer = New Customers
            _customer.CustomerID = customerID

            customerFactory.Load(_customer, 2, _cacheEnabled)

            _order = Nothing
            _orderNo = 0
            Return _customer
        Catch ex As Exception
            Dim s As String
            s = ex.Message
        End Try
        Return Nothing
    End Function
#End Region
#Region "Orders Related Operations"
    'Loads the orders of the customer from the cache if enabled otherwise from the database
    Private Function LoadCustomerOrders(ByVal cacheEnabled As Boolean) As Boolean
        Dim orderFactory As OrdersFactory
        Dim loaded As Boolean = False
        Try
            _order = Nothing
            orderFactory = New OrdersFactory
            _customer.Orders = orderFactory.getCustomerOrders(_customer.CustomerID, _cacheEnabled)
            If _customer.Orders.Items.Count > 0 Then
                _order = _customer.Orders.Items(0)
                loaded = True
            End If

        Catch ex As Exception
            Dim s As String
            s = ex.Message
        End Try
        Return loaded
    End Function
    'Gets the order of the customer. "orderNo" is the order to be loaded
    Public Function GetCustomerOrder(ByVal orderNo As Int32) As Orders
        If Not (_customer Is Nothing) Then
            If _customer.Orders Is Nothing Then
                If LoadCustomerOrders(_cacheEnabled) = False Then
                    Return Nothing
                End If
            End If
            If orderNo >= 0 And orderNo < OrdersCount Then
                Return CType(_customer.Orders.Items.Item(orderNo), Orders)
            End If
            Return Nothing
        End If
    End Function
    'Gets the first order of the customer
    Public Function GetFirstOrder() As Orders
        Dim order As Orders = GetCustomerOrder(0)
        If Not order Is Nothing Then
            _order = order
            _orderNo = 0
        End If
        Return order
    End Function
    'Gets the last order of the custoemr
    Public Function GetLastOrder() As Orders
        If _customer.Orders Is Nothing Then
            Return Nothing
        End If
        _orderNo = _customer.Orders.Count - 1
        _order = GetCustomerOrder(_orderNo)
        Return _order
    End Function
    'Gets the next order of the customer
    Public Function GetNextOrder() As Orders
        Dim order As Orders = GetCustomerOrder(_orderNo + 1)
        If Not order Is Nothing Then
            _orderNo += 1
            _order = order
        End If
        Return order
    End Function
    'Gets the previous order of the customer
    Public Function GetPreviousOrder() As Orders
        Dim order As Orders = GetCustomerOrder(_orderNo - 1)
        If Not order Is Nothing Then
            _orderNo -= 1
            _order = order
        End If
        Return order
    End Function
    'Updates the order changes in both database and cache as well
    Public Sub UpdateOrder()
        Dim orderFactory As OrdersFactory
        orderFactory = New OrdersFactory
        Dim objInfo As Customers = New Customers
        Try
            Dim orderInfo As Orders
            orderFactory.Save(_customer.Orders, 1, _cacheEnabled, New Customers)
        Catch ex As Exception
            Throw New Exception("order can not be saved")
        End Try

    End Sub
#End Region
#Region "Order Details Related Operations"
    'Gets the OrdersDetails collection  of the currents customer order
    Public Function GetCustomerOrderDetails() As OrderDetailsCollection
        If _order Is Nothing Then
            If LoadCustomerOrders(_cacheEnabled) = False Then
                Return Nothing
            End If
        End If
        'LoadCustomerOrdersDetails(_cacheEnabled)
        Return _order.OrderDetails
    End Function
    'Gets the detail of a specific product
    Public Function GetOrderDetails(ByVal prodId As Int64) As OrderDetails
        Dim orderDetail As OrderDetails
        If Not _order.OrderDetails Is Nothing Then
            For Each orderDetail In _order.OrderDetails
                If orderDetail.ProductID = prodId Then
                    Return orderDetail
                End If
            Next
        End If
        Return Nothing
    End Function
    'Updates the order details 
    Public Sub UpdateOrderDetails()
        Dim orderDetailFactory As OrderDetailsFactory
        Dim orderDetail As OrderDetails
        orderDetailFactory = New OrderDetailsFactory
        For Each orderDetail In _order.OrderDetails
            orderDetail.OrderID = _order.OrderID
        Next
        Try
            orderDetailFactory.Save(_order.OrderDetails, 0, _cacheEnabled, Nothing)
        Catch ex As Exception
            Throw New Exception("order can not be saved")
        End Try
    End Sub
#End Region
#Region "Products Related Operations"
    'Loads all the products from the cache if enabled otherwise from database
    Private Sub LoadProducts(ByVal cacheEnabled As Boolean)
        Dim productFactory As ProductsFactory
        Try
            productFactory = New ProductsFactory
            _products = productFactory.getAll(_cacheEnabled)
        Catch ex As Exception
            Dim s As String
            s = ex.Message
        End Try
    End Sub
    'Gets the product. 
    '"prodId" is the id of the product to be get
    Public Function GetProduct(ByVal prodId As Int32) As Products
        Dim productFactory As ProductsFactory
        Dim product As Products
        Try
            product = New Products
            product.ProductID = prodId
            productFactory = New ProductsFactory
            productFactory.Load(product, 0, _cacheEnabled)
           
        Catch ex As Exception
            Dim s As String
            s = ex.Message
            Return Nothing
        End Try
        Return product
    End Function
    'Updates the product collection
    Public Sub UpdateProducts(ByRef product As Products)
        Dim prod As Products
        Dim index = -1
        If Not Me._products Is Nothing Then
            For Each prod In _products
                If prod.ProductID = product.ProductID Then
                    index = _products.IndexOf(prod)
                End If
            Next
            If index >= 0 Then
                _products(index) = product
            End If
        End If
    End Sub
    'Gets the product information already loaded either form cahce or database
    Public Function GetProductLocally(ByVal productId As Integer) As Products
        Dim product As Products
        If Not _products Is Nothing Then
            For Each product In _products
                If product.ProductID = productId Then
                    Return product
                End If
            Next
        End If
        Return Nothing
    End Function
    'Gets the list of the produsts 
    Public Function GetProductList() As ArrayList
        Dim ie As IEnumerator
        Dim productList As ArrayList
        LoadProducts(_cacheEnabled)
        If Not (_products Is Nothing) Then
            ie = _products.GetEnumerator()
            productList = New ArrayList
            While ie.MoveNext
                productList.Add(CType(ie.Current, Products))
            End While
        End If

        Return productList
    End Function
    'Updates the product 
    Public Sub UpdateProduct()
        Dim productFactory As ProductsFactory
        productFactory = New ProductsFactory
        productFactory.Save(_products, 0, _cacheEnabled)
    End Sub
#End Region
#Region "Sales Person Related Operations"
    'Gets the list of the sales persons
    Public Function GetSalesPersonsList() As ArrayList
        Dim employeeFactory As EmployeesFactory
        Dim ie As IEnumerator
        Dim employeeList As ArrayList
        Dim employee As Employees
        Try
            employeeList = New ArrayList
            employeeFactory = New EmployeesFactory

            _salesPerson = employeeFactory.getAll(_cacheEnabled)
            ie = _salesPerson.GetEnumerator()
            While ie.MoveNext
                employee = CType(ie.Current, Employees)
                employeeList.Add(New SalesPerson(employee.EmployeeID, employee.FirstName))
            End While
        Catch ex As Exception

        End Try
        Return employeeList
    End Function
#End Region
#Region "Utility Operations"
    'Remove item from a new order 
    Public Sub RemoveItem(ByVal orderDetail As OrderDetails)
        If _order.OrderDetails.Contains(orderDetail) Then
            _order.OrderDetails.Remove(orderDetail)
        End If

    End Sub
    'Creates a new order for the customer
    Public Function CreateNewOrder() As Boolean
        Dim order As Orders = New Orders
        Dim shipDate = DateTime.Today.AddDays(1)
        Dim reqDate = DateTime.Today.AddDays(2)


        order.ShipAddress = _customer.Address
        order.ShipCity = _customer.City
        order.ShipPostalCode = _customer.PostalCode
        order.ShipCountry = _customer.Country
        order.ShipVia = 1
        order.ShipName = _customer.ContactName
        order.OrderDate = Date.Today
        order.ShippedDate = shipDate
        order.RequiredDate = reqDate
        order.CustomerID = _customer.CustomerID
        order.EmployeeID = _salesPerson.item(0).EmployeeID
        order.Freight = 0.0
        order.OrderDetails = New OrderDetailsCollection

        If _customer.Orders Is Nothing Then
            _customer.Orders = New OrdersCollection
        End If
        _customer.Orders.Add(order)
        _order = order
        _orderNo = OrdersCount - 1
        Return True
    End Function
    'Loads the all information about  three customer 
    Public Sub LoadAllInCache()
        Me._customerCollection = New CustomersCollection
        Dim customerFactory As CustomersFactory
        Dim Customer As Customers

        NCacheUtil.DisposeCache()
        NCacheUtil.InitCache(_inproc, UseCache)

        customerFactory = New CustomersFactory
        Customer = New Customers
        Customer.CustomerID = "ANATR"
        customerFactory.Load(Customer, 2, True)
        _customerCollection.Add(Customer)

        Customer = New Customers
        Customer.CustomerID = "ERNSH"
        customerFactory.Load(Customer, 2, True)
        _customerCollection.Add(Customer)

        Customer = New Customers
        Customer.CustomerID = "BERGS"
        customerFactory.Load(Customer, 2, True)
        _customerCollection.Add(Customer)

        Me.GetSalesPersonsList()
        LoadProducts(True)
    End Sub
    'Clears the cache
    Public Sub ClearCache()
        NCacheUtil.ClearCache()
    End Sub
#End Region
End Class
#End Region